"""Provide constants for the enhancement utility module."""

BRG_MAX = 255.0
CLR_MAX = 255.0
CMYK_MAX = 100.0
HUE_MAX = 360.0
HSV_MAX = 100.0

CM_BGR = "BGR"
CM_RGB = "RGB"
CM_CMYK = "CMYK"
CM_GRAY = "GRAY"
CM_HSV = "HSV"

UNNAMED_TABLE = "UNNAMED"
